"""Integration testing for pytha_quad function
"""

import pytest
from typing import List
from fakeproj.gooddir.goodmodule import pytha_quad
from fakeproj.fakedir.fakemodule import fakefunc


@pytest.mark.goodmodule
@pytest.mark.integration
@pytest.mark.parametrize(
    "numbers",
    [
        [1, 2, 2, 3],
        [2, 3, 6, 7],
        [6, 6, 7, 11],
        [2, 6, 9, 11],
    ],
)
def test_pytha_quad_integ(numbers: List[int]) -> None:
    """Integration Pass Test for pytha_quad

    Parameters
    ----------
    numbers : List[int]
        set of 4 numbers
    """
    np_numbers = fakefunc(numbers)
    list_numbers = np_numbers.tolist()
    assert list_numbers == numbers
    assert pytha_quad(list_numbers)



"""Test SlowStuff methods
"""

import pytest
from fakeproj.slowdir.slowmodule import SlowStuff


@pytest.mark.unit
@pytest.mark.slow
@pytest.mark.parametrize(
    "wait_time",
    [0.5, 1.0],
)
@pytest.mark.parametrize(
    "num_elements",
    [2, 5],
)
@pytest.mark.parametrize(
    "small_dict",
    [{"A": 1, "B": 2, "C": 3}, dict(zip([range(1, 10, 1)], [range(100, 1000, 100)]))],
)
def test_SlowStuff(wait_time: float, num_elements: int, small_dict: dict) -> None:
    """test the 3 methods of SlowStuff class

    Parameters
    ----------
    wait_time : float
        time to wait between iterations
    num_elements : int
        number of elements in iterables
    small_dict : dict
        pre-made sample dictionary
    """
    slow_obj = SlowStuff(0.5, 10)

    assert slow_obj.slow_dict()
    assert slow_obj.slow_list()
    assert slow_obj.dict_setup(small_dict)



"""Functions attempting to do borderline useful stuff
"""
from typing import List

Vector = List[float]


def funky_sum(numbers: Vector) -> float:
    """return the sum of a list offset by 42

    Parameters
    ----------
    numbers : list[float]
        list of numbers

    Returns
    -------
    float
        sum offset by 42
    """
    return sum(numbers) + 42


def recp_sum(numbers: Vector) -> float:
    """calculate reciprocal sum of a list of numbers

    Parameters
    ----------
    numbers : list[float]
        list of numbers

    Returns
    -------
    float
        reciprocal sum
    """
    result = 0.0
    for i in numbers:
        if i != 0:
            result += 1.0 / i
    return result


def pytha_quad(numbers: List[int]) -> bool:
    """check if a list is pythagorean quadruplet

    Parameters
    ----------
    numbers : List[int]
        list of 4 numbers

    Returns
    -------
    bool
        if it is a pythagorean quadruplet
    """
    assert len(numbers) == 4, "Should be a list of 4 numbers"

    sq_numbers = [i * i for i in numbers[:-1]]
    return sum(sq_numbers) == (numbers[-1] * numbers[-1])



"""module for testing pythagorean quadruplets function
"""

import pytest
from fakeproj.gooddir.goodmodule import pytha_quad
from typing import List


@pytest.mark.unit
@pytest.mark.goodmodule
@pytest.mark.parametrize(
    "numbers",
    [
        [1, 2, 2, 3],
        [2, 3, 6, 7],
        [4, 4, 7, 9],
        [1, 4, 8, 9],
        [6, 6, 7, 11],
        [2, 6, 9, 11],
    ],
)
def test_pytha_quad(numbers: List[int]) -> None:
    """pass test for pythagorean quads

    Parameters
    ----------
    numbers : List[int]
        set of 4 numbers
    """
    assert pytha_quad(numbers)


@pytest.mark.unit
@pytest.mark.goodmodule
@pytest.mark.parametrize("numbers", [[1, 1, 1, 1], [1, 2, 3, 4], [5, 6, 7, 8]])
def test_not_pytha_quad(numbers: List[int]) -> None:
    """fail test for pythagorean quads

    Parameters
    ----------
    numbers : List[int]
        set of 4 numbers
    """
    assert not pytha_quad(numbers)



"""
module to test fake functions
"""

from fakeproj.fakedir.fakemodule import fakefunc
import pytest


@pytest.mark.unit
@pytest.mark.fakemodule
def test_dummy_func() -> None:
    """test dummy_func()"""
    mylist = [1, 2, 3, 4, 5]
    assert fakefunc(mylist).tolist() == mylist



"""fake module with two fake functions used for tests
"""
import numpy as np  # type: ignore
from typing import List


def fakefunc(fakelist: List[int]) -> np.array:
    """take a list and return numpy array

    Parameters
    ----------
    fakelist : list[int]
        list of ints

    Returns
    -------
    numpy.array
        numpy array from python list
    """
    fake_array = np.array(fakelist)
    print(fake_array)
    return fake_array


def yaff(fakestr: str) -> str:
    """yet another fake function to print and return

    Parameters
    ----------
    fakestr : str
        any random string

    Returns
    -------
    str
        return the same string
    """
    print(fakestr)
    return fakestr



"""module for testing get_ssh()
"""

from fakeproj.gooddir.sysmodule import get_ssh
import pytest
import os
import platform


@pytest.mark.unit
@pytest.mark.sysmodule
@pytest.mark.skipif(platform.system() == "Windows", reason="fails on Windows")
def test_get_ssh_linux(monkeypatch) -> None:
    """test get_ssh on Linux

    Parameters
    ----------
    monkeypatch : MonkeyPatch
        set os.environ['Home'] to a fixed value
    """
    monkeypatch.setitem(os.environ, "HOME", "/Baker/Street/221B")
    assert get_ssh() == "/Baker/Street/221B/.ssh"


@pytest.mark.unit
@pytest.mark.sysmodule
@pytest.mark.skipif(
    platform.system() == "Linux" or platform.system() == "Darwin",
    reason="fails on Linux/macOS",
)
def test_get_ssh_windows(monkeypatch) -> None:
    """test get_ssh on Windows

    Parameters
    ----------
    monkeypatch : MonkeyPatch
        set os.environ['Home'] to a fixed value
    """
    monkeypatch.setitem(os.environ, "HOME", "C:\\You\\In\\A\\While")
    assert get_ssh() == "C:\\You\\In\\A\\While\\.ssh"



"""
module to test yet another fake functions
"""
import pytest
from fakeproj.fakedir.fakemodule import yaff


@pytest.mark.unit
@pytest.mark.fakemodule
def test_yaff() -> None:
    """test yaff"""
    assert yaff("foo") == "foo"



"""Attempting to do things very slowly
"""

import time


class SlowStuff:
    """
    methods for doing things really slowly
    """

    def __init__(self, wait_time: float, num_elements: int) -> None:
        """Constructor for SlowStuff

        Parameters
        ----------
        wait_time : float
            seconds to wait between iteration steps
        num_elements : int
            number of elements in the iterable
        """
        self.wait_time = wait_time
        self.num_elements = num_elements

    def slow_list(self) -> bool:
        """create a list and iterate slowly

        Returns
        -------
        bool
            True if the iteration completes
        """
        big_list = [range(self.num_elements)]

        for ele in big_list:
            time.sleep(self.wait_time)

        return True

    def slow_dict(self) -> bool:
        """create a dictionary and iterate slowly

        Returns
        -------
        bool
            True if the iteration completes
        """
        list_1 = [range(1, self.num_elements, 1)]
        list_2 = [range(1000, self.num_elements, 100)]
        big_dict = dict(zip(list_1, list_2))

        for key, value in big_dict.items():
            time.sleep(self.wait_time)

        return True

    def dict_setup(self, pre_made_dict: dict) -> bool:
        """iterate slowly through the supplied dictionary

        Parameters
        ----------
        pre_made_dict : dict
            dictionary of random values

        Returns
        -------
        bool
            True if the iteration completes
        """
        for key, value in pre_made_dict.items():
            time.sleep(self.wait_time)

        return True



"""module for testing sysmodule library
"""

from fakeproj.gooddir.sysmodule import get_new_path
import pytest
import os
import platform


@pytest.mark.unit
@pytest.mark.sysmodule
@pytest.mark.skipif(platform.system() == "Windows", reason="fails on Windows")
def test_get_new_path_linux(monkeypatch) -> None:
    """test get_new_path on linux

    Parameters
    ----------
    monkeypatch : MonkeyPatch
        monkeypatch the os.getcwd()
    """

    def mock_getcwd() -> str:
        """always return a mock path "/beam/me/up"

        Returns
        -------
        str
            mock path
        """
        return "/beam/me/up"

    monkeypatch.setattr(os, "getcwd", mock_getcwd)
    assert get_new_path("scotty") == "/beam/me/up/scotty"


@pytest.mark.sysmodule
@pytest.mark.skipif(
    platform.system() == "Linux" or platform.system() == "Darwin",
    reason="fails on Linux/macOS",
)
def test_get_new_path_win(monkeypatch) -> None:
    """test get_new_path on Windows

    Parameters
    ----------
    monkeypatch : MonkeyPatch
        monkeypatch the os.getcwd()
    """

    def mock_getcwd() -> str:
        """always return a mock path "C:\\You\\Later"

        Returns
        -------
        str
            mock path
        """
        return "C:\\You\\Later"

    monkeypatch.setattr(os, "getcwd", mock_getcwd)
    assert get_new_path("Alligator") == "C:\\You\\Later\\Alligator"



"""module for testing reciprocal sum
"""
import pytest
from test.conftest import data_sample_list  # noqa F401
from fakeproj.gooddir.goodmodule import recp_sum


@pytest.mark.unit
@pytest.mark.goodmodule
def test_recp(data_sample_list) -> None:  # noqa F811
    """test reciprocal sum on [1, 2, 4]"""
    assert recp_sum(data_sample_list) == 1.75



"""module for testing funky_sum
"""
from test.conftest import data_sample_list  # noqa F401
from fakeproj.gooddir.goodmodule import funky_sum
import pytest


@pytest.mark.unit
@pytest.mark.goodmodule
def test_funky(data_sample_list) -> None:  # noqa F811
    """test funky sum with [1, 2, 4]"""
    assert funky_sum(data_sample_list) == 49



"""
setup file for installing package
"""

from setuptools import setup

setup(
    name="fakeproj",
    version="1.0",
    url="https://github.com/lazyoracle/fakeproj",
    author="Anurag Saha Roy",
    author_email="anurag.roni@gmail.com",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Build Tools",
    ],
    include_package_data=True,
    packages=["fakeproj", "fakeproj/fakedir", "fakeproj/gooddir", "fakeproj/slowdir"],
    description="Mostly useless code to demonstrate Python DevOps tools",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    install_requires=["numpy==1.19.0"],
    python_requires=">=3.6",
)



"""library of functions for sys derived tasks
"""
import os


def get_new_path(dir_name: str) -> str:
    """return new path with dir name

    Parameters
    ----------
    dir_name : str
        name of new directory to be made

    Returns
    -------
    str
        path to new directory
    """
    cwd = os.getcwd()
    new_path = os.path.join(cwd, dir_name)
    return new_path


def get_ssh() -> str:
    """return path to .ssh directory

    Returns
    -------
    str
        path to .ssh
    """
    home_dir = os.environ["HOME"]
    ssh_dir = os.path.join(home_dir, ".ssh")
    return ssh_dir
